=== modified file 'ivle/webapp/tutorial/__init__.py'
--- ivle/webapp/tutorial/__init__.py	2009-02-22 07:27:29 +0000
+++ ivle/webapp/tutorial/__init__.py	2009-02-22 13:44:08 +0000
@@ -46,7 +46,7 @@
 from ivle.webapp.errors import NotFound, Forbidden
 from ivle.webapp.tutorial.rst import rst as rstfunc
 from ivle.webapp.tutorial.service import AttemptsRESTView, AttemptRESTView, \
-                                         ExerciseRESTView, WorksheetRESTView
+                        ExerciseRESTView, WorksheetRESTView, OfferingRESTView
 
 class Worksheet:
     """This class represents a worksheet and a particular students progress
@@ -377,16 +377,21 @@
     worksheets are actually displayed on the page."""
     pass
 
-class WorksheetAdminView(XHTMLView):
+class WorksheetEditView(XHTMLView):
     """The admin view for an offering.
     
     This view is designed to replace worksheets.xml, turning them instead
     into XML directly from RST."""
     permission = "edit"
-    template = "worksheet_admin.html"
-    appname = "Worksheet Admin"
+    template = "worksheet_edit.html"
+    appname = "Edit Worksheet"
 
-    def __init__(self, req, subject, year, semester, worksheet):
+    def __init__(self, req, **kwargs):
+    
+        subject = kwargs['subject']
+        year = kwargs['year']
+        semester = kwargs['semester']
+        worksheet = kwargs['worksheet']
         self.context = req.store.find(DBWorksheet,
             DBWorksheet.identifier == worksheet,
             DBWorksheet.offering_id == Offering.id,
@@ -397,13 +402,14 @@
             Subject.code == subject
         ).one()
         
+        if self.context is None:
+            raise NotFound()
+        
         self.subject = subject
         self.year = year
         self.semester = semester
         self.worksheet = worksheet
         
-        if self.context is None:
-            raise NotFound()
             
     def populate(self, req, ctx):
         self.plugin_styles[Plugin] = ["tutorial_admin.css"]
@@ -416,13 +422,48 @@
         ctx['semester'] = self.semester
 
 
+class WorksheetAddView(XHTMLView):
+    """This view allows a user to add a worksheet"""
+    permission = "edit"
+    template = "worksheet_add.html"
+    appname = "Add Worksheet"
+
+    def __init__(self, req, subject, year, semester):
+        self.context = req.store.find(Offering,
+            Offering.semester_id == Semester.id,
+            Semester.year == year,
+            Semester.semester == semester,
+            Offering.subject_id == Subject.id,
+            Subject.code == subject
+        ).one()
+        
+        self.subject = subject
+        self.year = year
+        self.semester = semester
+        
+        if self.context is None:
+            raise NotFound()
+            
+    def populate(self, req, ctx):
+        self.plugin_styles[Plugin] = ["tutorial_admin.css"]
+        self.plugin_scripts[Plugin] = ['tutorial_admin.js']
+        
+        ctx['subject'] = self.subject
+        ctx['year'] = self.year
+        ctx['semester'] = self.semester
+        
+        #XXX: Get the list of formats from somewhere else
+        ctx['formats'] = ['xml', 'rst']
+
+
 class Plugin(ViewPlugin, MediaPlugin):
     urls = [
         ('subjects/:subject/:year/:semester/+worksheets', OfferingView),
-        ('subjects/:subject/:year/:semester/+worksheets/+edit', OfferingAdminView),
+        ('subjects/:subject/:year/:semester/+worksheets/+add', WorksheetAddView),
         ('subjects/:subject/+worksheets/+media/*(path)', SubjectMediaView),
         ('subjects/:subject/:year/:semester/+worksheets/:worksheet', WorksheetView),
-        ('subjects/:subject/:year/:semester/+worksheets/:worksheet/+edit', WorksheetAdminView),
+        ('subjects/:subject/:year/:semester/+worksheets/:worksheet/+edit', WorksheetEditView),
+        ('api/subjects/:subject/:year/:semester/+worksheets', OfferingRESTView),
         ('api/subjects/:subject/:year/:semester/+worksheets/:worksheet/*exercise/'
             '+attempts/:username', AttemptsRESTView),
         ('api/subjects/:subject/:year/:semester/+worksheets/:worksheet/*exercise/'

=== modified file 'ivle/webapp/tutorial/media/tutorial_admin.js'
--- ivle/webapp/tutorial/media/tutorial_admin.js	2009-02-19 05:47:56 +0000
+++ ivle/webapp/tutorial/media/tutorial_admin.js	2009-02-22 13:44:08 +0000
@@ -22,7 +22,7 @@
  * Refreshes the page.
  */
 
-function submitworksheet()
+function editworksheet()
 {
     /* Get the worksheet code the admin is submitting */
     var namearea = document.getElementById("worksheet_name");
@@ -59,3 +59,47 @@
         "/+worksheets/" + worksheet;
     ajax_call(callback, save_path, "", args, "POST");
 }
+
+function addworksheet()
+{
+    /* Get the worksheet code the admin is submitting */
+    var idarea = document.getElementById("worksheet_identifier");
+    var namearea = document.getElementById("worksheet_name");
+    var check_box = document.getElementById("worksheet_asses");
+    var sheet_area = document.getElementById("worksheet_data");
+    var format_opt = document.getElementById("worksheet_format");
+    
+    var worksheet_identifier = idarea.value;
+    var worksheetname = namearea.value;
+    var assessable = check_box.checked;
+    var data = sheet_area.value;
+    var format = format_opt.value;
+
+    var args = {'ivle.op': 'add_worksheet', 'data': data, 'assessable': 
+        assessable, 'name': worksheetname, 'identifier': worksheet_identifier,
+        'format': format};
+
+    /* Send the form as multipart/form-data, since we are sending a whole lump
+     * of Python code, it should be treated like a file upload. */
+    /* AJAX callback function */
+    var callback = function(xhr)
+        {
+            var testresponse;
+            try
+            {
+                testresponse = JSON.parse(xhr.responseText);
+                alert("Worksheet successfully saved.");
+                window.location = "./";
+            }
+            catch (ex)
+            {
+                alert("There was an error submitting your worksheet. "
+                    + "Please notify the administrators of this.");
+                return;
+            }
+        }
+    //XXX: This shouldn't be generated here
+    save_path = "api/subjects/" + subject + "/" + year + "/" + semester +
+                "/+worksheets";
+    ajax_call(callback, save_path, "", args, "POST");
+}

=== modified file 'ivle/webapp/tutorial/service.py'
--- ivle/webapp/tutorial/service.py	2009-02-22 07:27:29 +0000
+++ ivle/webapp/tutorial/service.py	2009-02-22 13:44:08 +0000
@@ -130,10 +130,10 @@
         if user is None:
             raise NotFound()
 
-        try:
-            date = datetime.datetime.strptime(date, TIMESTAMP_FORMAT)
-        except ValueError:
-            raise NotFound()
+        #try:
+        #    date = datetime.datetime.strptime(date, TIMESTAMP_FORMAT)
+        #except ValueError:
+        #    raise NotFound()
 
         worksheet_exercise = req.store.find(WorksheetExercise,
             WorksheetExercise.exercise_id == exercise,
@@ -147,10 +147,10 @@
             Semester.semester == semester).one()
             
         attempt = req.store.find(ExerciseAttempt,
-            ExerciseAttempt.login_id == user.id,
+            ExerciseAttempt.user_id == user.id,
             ExerciseAttempt.ws_ex_id == worksheet_exercise.id,
             ExerciseAttempt.date == date
-        )
+        ).one()
 
         if attempt is None:
             raise NotFound()
@@ -179,7 +179,7 @@
         # Find the appropriate WorksheetExercise to save to. If its not found,
         # the user is submitting against a non-existant worksheet/exercise
         worksheet_exercise = req.store.find(WorksheetExercise,
-            WorksheetExercise.exericise_id == self.exercise,
+            WorksheetExercise.exercise_id == self.exercise,
             WorksheetExercise.worksheet_id == Worksheet.id,
             Worksheet.offering_id == Offering.id,
             Offering.subject_id == Subject.id,
@@ -191,64 +191,77 @@
         if worksheet_exercise is None:
             raise NotFound()
 
-        new_save = ExerciseSave()
+        old_save = req.store.find(ExerciseSave,
+            ExerciseSave.ws_ex_id == worksheet_exercise.id,
+            ExerciseSave.user == req.user).one()
+        
+        #Overwrite the old, or create a new if there isn't one
+        if old_save is None:
+            new_save = ExerciseSave()
+            req.store.add(new_save)
+        else:
+            new_save = old_save
+        
         new_save.worksheet_exercise = worksheet_exercise
         new_save.user = req.user
         new_save.text = unicode(text)
         new_save.date = datetime.datetime.now()
-        req.store.add(new_save)
+
         return {"result": "ok"}
 
 
+def generate_exerciselist(worksheet, req, worksheetdata):
+    """Runs through the worksheetstream, generating the appropriate
+    WorksheetExercises, and de-activating the old ones."""
+    exercises = []
+    # Turns the worksheet into an xml stream, and then finds all the 
+    # exercise nodes in the stream.
+    worksheetdata = genshi.XML(worksheetdata)
+    for kind, data, pos in worksheetdata:
+        if kind is genshi.core.START:
+            # Data is a tuple of tag name and a list of name->value tuples
+            if data[0] == 'exercise':
+                src = ""
+                optional = False
+                for attr in data[1]:
+                    if attr[0] == 'src':
+                        src = attr[1]
+                    if attr[0] == 'optional':
+                        optional = attr[1] == 'true'
+                if src != "":
+                    exercises.append((src, optional))
+    ex_num = 0
+    # Set all current worksheet_exercises to be inactive
+    db_worksheet_exercises = req.store.find(WorksheetExercise,
+        WorksheetExercise.worksheet_id == worksheet.id)
+    for worksheet_exercise in db_worksheet_exercises:
+        worksheet_exercise.active = False
+    
+    for exerciseid, optional in exercises:
+        worksheet_exercise = req.store.find(WorksheetExercise,
+            WorksheetExercise.worksheet_id == worksheet.id,
+            Exercise.id == WorksheetExercise.exercise_id,
+            Exercise.id == exerciseid).one()
+        if worksheet_exercise is None:
+            exercise = req.store.find(Exercise,
+                Exercise.id == exerciseid
+            ).one()
+            if exercise is None:
+                raise NotFound()
+            worksheet_exercise = WorksheetExercise()
+            worksheet_exercise.worksheet_id = worksheet.id
+            worksheet_exercise.exercise_id = exercise.id
+            req.store.add(worksheet_exercise)
+        worksheet_exercise.active = True
+        worksheet_exercise.seq_no = ex_num
+        worksheet_exercise.optional = optional
+
+
+# Note that this is the view of an existing worksheet. Creation is handled
+# by OfferingRESTView (as offerings have worksheets)
 class WorksheetRESTView(JSONRESTView):
     """View used to update a worksheet."""
 
-    def generate_exerciselist(self, req, worksheet):
-        """Runs through the worksheetstream, generating the appropriate
-        WorksheetExercises, and de-activating the old ones."""
-        exercises = []
-        # Turns the worksheet into an xml stream, and then finds all the 
-        # exercise nodes in the stream.
-        worksheet = genshi.XML(worksheet)
-        for kind, data, pos in worksheet:
-            if kind is genshi.core.START:
-                # Data is a tuple of tag name and a list of name->value tuples
-                if data[0] == 'exercise':
-                    src = ""
-                    optional = False
-                    for attr in data[1]:
-                        if attr[0] == 'src':
-                            src = attr[1]
-                        if attr[0] == 'optional':
-                            optional = attr[1] == 'true'
-                    if src != "":
-                        exercises.append((src, optional))
-        ex_num = 0
-        # Set all current worksheet_exercises to be inactive
-        db_worksheet_exercises = req.store.find(WorksheetExercise,
-            WorksheetExercise.worksheet_id == self.context.id)
-        for worksheet_exercise in db_worksheet_exercises:
-            worksheet_exercise.active = False
-        
-        for exerciseid, optional in exercises:
-            worksheet_exercise = req.store.find(WorksheetExercise,
-                WorksheetExercise.worksheet_id == self.context.id,
-                Exercise.id == WorksheetExercise.exercise_id,
-                Exercise.id == exerciseid).one()
-            if worksheet_exercise is None:
-                exercise = req.store.find(Exercise,
-                    Exercise.id == exerciseid
-                ).one()
-                if exercise is None:
-                    raise NotFound()
-                worksheet_exercise = WorksheetExercise()
-                worksheet_exercise.worksheet_id = self.context.id
-                worksheet_exercise.exercise_id = exercise.id
-                req.store.add(worksheet_exercise)
-            worksheet_exercise.active = True
-            worksheet_exercise.seq_no = ex_num
-            worksheet_exercise.optional = optional
-
     def get_permissions(self, user):
         # XXX: Do it properly.
         # XXX: Lecturers should be allowed to add worksheets Only to subjects
@@ -281,12 +294,63 @@
             raise NotFound()
     
     @named_operation('save')
-    def save(self, req, name, assessable, data):
+    def save(self, req, name, assessable, data, format):
         """Takes worksheet data and saves it."""
-        self.generate_exerciselist(req, data)
+        self.generate_exerciselist(self.context, req, data)
         
         self.context.name = unicode(name)
+        self.context.assessable = self.convert_bool(assessable)
         self.context.data = unicode(data)
-        self.context.assessable = self.convert_bool(assessable)
-        
-        return {"result": "ok"}
+        
+        return {"result": "ok"}
+
+class OfferingRESTView(JSONRESTView):
+    """View used to update Offering and create Worksheets."""
+    
+    def get_permissions(self, user):
+        # XXX: Do it properly.
+        # XXX: Lecturers should be allowed to add worksheets Only to subjects
+        #      under their control
+        if user is not None:
+            if user.rolenm == 'admin':
+                return set(['add_worksheet'])
+            else:
+                return set()
+        else:
+            return set()
+
+    def __init__(self, req, **kwargs):
+    
+        self.subject = kwargs['subject']
+        self.year = kwargs['year']
+        self.semester = kwargs['semester']
+    
+        self.context = req.store.find(Offering,
+            Offering.subject_id == Subject.id,
+            Subject.code == self.subject,
+            Offering.semester_id == Semester.id,
+            Semester.year == self.year,
+            Semester.semester == self.semester).one()
+        
+        if self.context is None:
+            raise NotFound()
+
+    @named_operation('add_worksheet')
+    def add_worksheet(self,req, identifier, name, assessable, data, format):
+        """Takes worksheet data and adds it."""
+        
+        new_worksheet = Worksheet()
+        
+        new_worksheet.offering = self.context
+        new_worksheet.identifier = unicode(identifier)
+        new_worksheet.name = unicode(name)
+        new_worksheet.assessable = self.convert_bool(assessable)
+        new_worksheet.data = unicode(data)
+        new_worksheet.format = unicode(format)
+        new_worksheet.seq_no = self.context.worksheets.count()
+        req.store.add(new_worksheet)
+        
+        generate_exerciselist(new_worksheet, req, data)
+        
+        return {"result": "ok"}
+

=== added file 'ivle/webapp/tutorial/worksheet_add.html'
--- ivle/webapp/tutorial/worksheet_add.html	1970-01-01 00:00:00 +0000
+++ ivle/webapp/tutorial/worksheet_add.html	2009-02-22 13:44:08 +0000
@@ -0,0 +1,42 @@
+<html xmlns="http://www.w3.org/1999/xhtml"
+      xmlns:py="http://genshi.edgewall.org/">
+  <head>
+    <title>Tutorial - Worksheet Editor</title>
+    <!-- These let the javascript know which worksheet is open -->
+    <script type="text/javascript">
+        subject = "${subject}";
+        year = "${year}";
+        semester = "${semester}";
+    </script>
+  </head>
+  <body>
+    <div id="ivle_padding">
+      <h1>IVLE Worksheet - ${subject} - ${year} - Semester ${semester}</h1>
+      <div id='worksheet_input'>
+        <label for='worksheet_identifier'>Unique Identifier:</label>
+        <input type='text' id='worksheet_identifier' class='worksheet_id' />
+        <br />
+        <label for='worksheet_name'>Name:</label>
+        <input type='text' id='worksheet_name' class='worksheet_area' />
+        <br />
+        <label for='worksheet_asses'>Assessable:</label>
+        <input type='checkbox' name='asses' id='worksheet_asses' />
+        <br />
+        <label for='worksheet_format'>Worksheet Format:</label>
+        <select name='format' id='worksheet_format'>
+          <py:for each="format in formats">
+            <option value='${format}'>${format}</option>
+          </py:for>
+        </select>
+        <br />
+        <textarea id='worksheet_data' class='worksheet_area' cols='80'
+        rows='30'></textarea>
+        <p>
+          <span id="savepanel">
+            <input onclick='addworksheet()' type="submit" value="Create" />
+          </span>
+        </p>
+      </div>
+    </div>
+  </body>
+</html>

=== renamed file 'ivle/webapp/tutorial/worksheet_admin.html' => 'ivle/webapp/tutorial/worksheet_edit.html'
